##########################################################################
# έp (@)                                                            #
# Anderson et al., Statistics for Business & Economics (14th Edition)    #
# Chapter 14: Simple Linear Regression                                   #
# d~ ߬FvjǲέpǨt                                            #
# http://www.hmwu.idv.tw/                                                #
##########################################################################

##########################################################################
# 14.2 Least Squares Method                                              #
##########################################################################

# TABLE 14.1 
# Student Population and Quarterly Sales Data for 10 Armands Pizza Parlors

library(readxl)
Armand <- read_excel("data/Chap14/Armand's.xlsx")
print(Armand)


# FIGURE 14.3 
# Scatter Diagram of Student Population and Quarterly Sales for Armands Pizza Parlors
attach(Armand)
plot(Population, Sales, col = "red", pch = 16,
     xlab = "Student Population (1000s)", 
     ylab = "Quarterly Sales ($1000s)", 
     main = "Scatter Diagram of Student Population and Quarterly Sales")



# FIGURE 14.4 
# Graph of the Estimated Regression Equation for Armands Pizza Parlors: yhat = 60 + 5x

Armand.lm <- lm(Sales ~ Population, data = Armand)
summary(Armand.lm)

plot(Population, Sales, col = "red", pch = 16,
     xlab = "Student Population (1000s)", 
     ylab = "Quarterly Sales ($1000s)", 
     main = "Graph of the Estimated Regression Equation for Armands Pizza Parlors")
abline(Armand.lm, col = "darkgreen", lwd = 2)
text(20, 110, bquote(paste("Intercept ", b[0] == .(coef(Armand.lm)[1]))))
text(20, 100, bquote(paste("Slope ", b[1] == .(coef(Armand.lm)[2]))))


# predict quarterly sales for a restaurant to be located near a campus with 
# 16,000 students
predict(Armand.lm, list(Population = 16))



##########################################################################
# 14.3 Coefficient of Determination                                      #
##########################################################################

names(anova(Armand.lm))
SSR <- anova(Armand.lm)$"Sum Sq"[1] 
SSR
SSE <- anova(Armand.lm)$"Sum Sq"[2]
SSE
SST <- SSR + SSE
SST

names(summary(Armand.lm))
summary(Armand.lm)$r.squared
summary(Armand.lm)$adj.r.squared

# Correlation Coefficient
rxy <- sign(coef(Armand.lm)[2]) * sqrt(summary(Armand.lm)$r.squared) 
rxy



##########################################################################
# 14.5 Testing for Significance                                          #
##########################################################################

# Mean Square Error (estimate OF sigma^2)
MSE <- anova(Armand.lm)$"Mean Sq"[2]
MSE


# t Test: H0: beta1 = 0
Armand.lm.s <- summary(Armand.lm)
str(Armand.lm.s)
str(Armand.lm.s$coefficients)
Armand.lm.s$coefficients[2, "t value"]
Armand.lm.s$coefficients[2, "Pr(>|t|)"]
critical.value <- qt(0.005, nrow(Armand)-2)
critical.value


# Confidence Interval for beta1
b1 <- Armand.lm.s$coefficients[2, "Estimate"]
b1
sb1 <- Armand.lm.s$coefficients[2, "Std. Error"]
sb1
b1 + sb1 * c(critical.value, -critical.value)

confint(Armand.lm, level = 0.99)


# F Test
# TABLE 14.6 Anova Table for the Armands Pizza Parlors Problem
Armand.lm.anova <- anova(Armand.lm)
Armand.lm.anova
str(Armand.lm.anova)
Armand.lm.anova$`Pr(>F)`[1]


##########################################################################
# 14.6 Using the Estimated Regression Equation for Estimation and        #
#      Prediction                                                        #
##########################################################################


# Confidence and Prediction Intervals for the Mean Sales y 
# at Given Values of Student Population x
new.dat <- data.frame(Population = 10)
predict(Armand.lm, newdata = new.dat, interval = "confidence")
predict(Armand.lm, newdata = new.dat, interval = "prediction")


# FIGURE 14.9 
# Confidence and Prediction Intervals for Sales y at Given Values of Student Population x
plot(Population, Sales, col = "red", pch = 16,
     xlam = c(0, 26), ylim = c(0, 240),
     xlab = "Student Population (1000s)", 
     ylab = "Quarterly Sales ($1000s)", 
     main = "Confidence and Prediction Intervals for Sales y at Given Values of Student Population x")
abline(Armand.lm, col = "darkblue", lwd = 2)

myPopulation <- seq(2, 26, 0.5)
conf.interval <- predict(Armand.lm, newdata = data.frame(Population = myPopulation), 
                         interval = "confidence", level = 0.95)
pred.interval <- predict(Armand.lm, newdata = data.frame(Population = myPopulation), 
                         interval = "prediction", level = 0.95)

matlines(myPopulation, conf.interval[, 2:3], col = "lightblue", lty = 2, lwd = 2)
matlines(myPopulation, pred.interval[, 2:3], col = "black", lty = 1, ld = 2)



##########################################################################
# 14.8 Residual Analysis: Validating Model Assumptions                   #
##########################################################################

plot(Armand.lm, which = 1:6) 

# FIGURE 14.11
# Plot of the Residuals Against the Independent Variable x for Armands Pizza Parlors
plot(Armand$Population, Armand.lm$residuals, 
     main = "Plot of the Residuals Against the Independent Variable x")
abline(h = 0, lty = 2, lwd = 2, col = "gray")


# FIGURE 14.13 
# Plot of the Residuals Against the Predicted Values y for Armands Pizza Parlors
plot(Armand.lm$fitted.values, Armand.lm$residuals, 
     main = "Plot of the Residuals Against the Predicted Values y")
abline(h = 0, lty = 2, lwd = 2, col = "gray")



# TABLE 14.8 
# Computation of Standardized Residuals for Armands Pizza Parlors
Armand.std.res <- rstandard(Armand.lm)
Armand.std.res


# FIGURE 14.14 
# Plot of the Standardized Residuals Against the Independent Variable x for Armands Pizza Parlors
plot(Armand$Population, Armand.std.res, 
     main = "Plot of the Standardized Residuals Against the Independent Variable x")
abline(h = 0, lty = 2, lwd = 2, col = "gray")


# FIGURE 14.15 
# Normal Probability Plot for Armands Pizza Parlors

# Normal Scores For n = 10
pnorm(-1.55)
pnorm(1.55)
Normal.Score.n10 <- qnorm(seq(pnorm(-1.55), pnorm(1.55), length.out = 10))
Ordered.Std.Res <- sort(Armand.std.res)
plot(Normal.Score.n10, Ordered.Std.Res)
abline(a = 0, b = 1, col = "darkgreen", lwd = 2)

qqnorm(Armand.std.res)
qqline(Armand.std.res)



##########################################################################
# 14.9 Residual Analysis: Outliers and Influential Observations          #
##########################################################################

# leverage of observation i

dataT14.12 <- data.frame(x = c(10, 10, 15, 20, 20, 25, 70),
                         y = c(125, 130, 120, 115, 120, 110, 100))
dataT14.12.lm <- lm(y ~ x, data = dataT14.12)

hatvalues(dataT14.12.lm)

# or
leverage <- function(x){
  n <- length(x)
  xbar <- mean(x)
  hvalues <- 1/n + (x-xbar)^2/sum((x-xbar)^2)
  influential <- rep("", n) 
  influential[which(hvalues  > 6/n)] <- "*"
  data.frame(hvalues, influential)
}
leverage(dataT14.12$x)


detach(Armand)








