##########################################################################
# έp (@)                                                            #
# Anderson et al., Statistics for Business & Economics (14th Edition)    #
# Chapter 17: Time Series Analysis and Forecasting                       #
# d~ ߬FvjǲέpǨt                                            #
# http://www.hmwu.idv.tw/                                                #
##########################################################################



##########################################################################
# 17.1 Time Series Patterns                                              #
##########################################################################

library(readxl)

Gasoline <- read_excel("data/Chap17/Gasoline.xlsx")
Gasoline

GasolineRevised <- read_excel("data/Chap17/GasolineRevised.xlsx")
GasolineRevised

Bicycle <- read_excel("data/Chap17/Bicycle.xlsx")
Bicycle

Cholesterol <- read_excel("data/Chap17/Cholesterol.xlsx")
Cholesterol

Umbrella <- read_excel("data/Chap17/Umbrella.xlsx")
Umbrella

SmartphoneSales <- read_excel("data/Chap17/SmartphoneSales.xlsx")
SmartphoneSales

par(mfrow = c(3, 2))
plot(Sales ~ Week, data = Gasoline, type = "o", pch = 16, col = "darkgreen",
     main = "Gasoline Sales Time Series Plot",
     ylim = c(0, 100))

plot(Sales ~ Week, data = GasolineRevised, type = "o", pch = 16, col = "darkgreen",
     main = "Gasoline Sales Revised Time Series Plot")

plot(Sales ~ Year, data = Bicycle, type = "o", pch = 16, col = "darkgreen",
     main = "Bicycle Sales Time Series Plot")

plot(Revenue ~ Year, data = Cholesterol, type = "o", pch = 16, col = "darkgreen",
     main = "Cholesterol Revenue Times Series Plot",
     ylim = c(0, 120))

plot(Sales ~  Period, data = Umbrella, type = "o", pch = 16, col = "darkgreen",
     main = "Umbrella Sales Time Series Plot",
     ylim = c(0, 180))
                                       
plot(`Sales (1000s)` ~ c(1:nrow(SmartphoneSales)), data = SmartphoneSales,
     type = "o", pch = 16, col = "darkgreen",
     main = "Quarterly Smartphone Sales Time Series Plot",
     ylim = c(0, 9))
                                       



##########################################################################
# 17.2 Forecast Accuracy                                                 #
##########################################################################
library(forecast)
# ? ts
Gasoline.ts <- ts(Gasoline$Sales)
plot(Gasoline.ts)

Gasoline.fc.naive <- naive(Gasoline.ts, h = 1)
Gasoline.fc.naive$fitted
summary(Gasoline.fc.naive)
accuracy(Gasoline.fc.naive)




##########################################################################
# 17.3 Moving Averages and Exponential Smoothing                         #  
##########################################################################


library(TTR) # Technical Trading Rules package

Gasoline.fc.sma3 <- SMA(Gasoline.ts, n = 3)
Gasoline.fc.wma3 <- WMA(Gasoline.ts, n = 3)
Gasoline.fc.ema3 <- EMA(Gasoline.ts, n = 3)

accuracy(Gasoline.fc.sma3, Gasoline.ts[2:12])
accuracy(Gasoline.fc.wma3, Gasoline.ts[2:12]) #
accuracy(Gasoline.fc.ema3, Gasoline.ts[2:12]) # different from textbook
         
# FIGURE 17.7 
# Gasoline Sales Time Series Plot and Three-Week Moving Average Forecasts
plot(Gasoline.ts, type = "o", ylim = c(0, 100), 
     lwd = 2, col = "darkgreen", pch = 16,
     main = "Gasoline Sales Time Series Plot and \n Three-Week Moving Average Forecasts")
points(Gasoline.fc.sma3, type = "o", lwd = 2, col = "blue", pch = 16)
legend(8, 40, legend = c("Gasoline.ts", "SMA"), col = c("darkgreen", "blue"),
       lty = 1, lwd = 2)





##########################################################################
# 17.4 Trend Projection                                                  #
##########################################################################


# FIGURE 17.10 
# Trend Represented by a Linear Function for the Bicycle Sales Time Series
plot(Sales ~ Year, data = Bicycle, type = "o", pch = 16, col = "darkgreen",
     main = "Trend Represented by a Linear Function for \n 
     Bicycle Sales Time Series Plot")

Bicycle.lm <- lm(Sales ~ Year, data = Bicycle)
abline(Bicycle.lm, col = "blue")

summary(Bicycle.lm)
anova(Bicycle.lm)

accuracy(predict(Bicycle.lm), Bicycle$Sales)




# Nonlinear Trend Regression
Cholesterol <- read_excel("data/Chap17/Cholesterol.xlsx")
Cholesterol
Cholesterol.nlm <- lm(Revenue ~ Year + Year^2, data = Cholesterol)
summary(Cholesterol.nlm)
anova(Cholesterol.nlm)
accuracy(predict(Cholesterol.nlm), Cholesterol$Revenue)




##########################################################################
# 17.5 Seasonality and Trend                                             #
##########################################################################

# FIGURE 17.16 
# Umbrella Sales Time Series Plot
Umbrella <- read_excel("data/Chap17/Umbrella.xlsx")
Umbrella
Umbrella.ts <- ts(Umbrella$Sales, frequency = 4)
plot(Umbrella.ts, type = "o", pch = 16, 
     ylim = c(0, 180),
     main = "Umbrella Sales Time Series Plot")

Umbrella$Quarter <- factor(Umbrella$Quarter, levels = 4:1) 
# R: Q1 is base, so need to set levels = 4:1
# textbook: Q4 is base

Umbrella.lm <- lm(Sales ~ Quarter, data = Umbrella)
summary(Umbrella.lm)
anova(Umbrella.lm)




# Seasonality and Trend
# FIGURE 17.18 
# Smartphone Sales Time Series Plot
SmartphoneSales <- read_excel("data/Chap17/SmartphoneSales.xlsx")
SmartphoneSales
SmartphoneSales.ts <- ts(SmartphoneSales$`Sales (1000s)`, frequency = 4)

plot(SmartphoneSales.ts, type = "o", pch = 16, 
     ylim = c(0, 9),
     main = "Smartphone Sales Time Series Plot",
     xlab = "Year", ylab = "Sales (1000s)")

SmartphoneSales$Quarter <- factor(SmartphoneSales$Quarter, levels = 4:1) 
SmartphoneSales$Period <- 1:nrow(SmartphoneSales)
  
SmartphoneSales.lm <- lm(`Sales (1000s)` ~ Quarter + Period, data = SmartphoneSales)
summary(SmartphoneSales.lm)
anova(SmartphoneSales.lm)


# forecast quarterly sales for next year. Next year is
# year 5 for the smartphone sales time series; 
# that is, time periods 17, 18, 19, and 20.
predict(SmartphoneSales.lm, newdata = data.frame(Quarter = factor(1:4), Period = 17:20))


